@@ -51,7 +51,7 @@ impl App {
             if progress_interval == 0 {
                 activity.update_progress();
                 activity.run();
-                activity.update_playlist();
+                activity.update_download_progress();
             }
             progress_interval += 1;
             if progress_interval >= 8 {
