@@ -23,6 +23,9 @@ impl MainActivity {
     }
 
     pub fn sync_items(&mut self) {
+        if self.queue_items.is_empty() {
+            return;
+        }
         let mut table: TableBuilder = TableBuilder::default();
 
         for (idx, record) in self.queue_items.iter().enumerate() {
@@ -61,7 +64,6 @@ impl MainActivity {
                     )
                     .build(),
                 );
-            // table.add_col(TextSpan::from(format!("{}", record)));
         }
         let table = table.build();
 
